if(window.Nire === undefined) Nire = {};

Nire.Connection = function(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired) {
    this.log = getLogger("Connection");
	this.notLoggedInCallback = notLoggedInCallback;
    this.networkFailureAlert = networkFailureAlert;
    this.warnUserAppUpdateRequired = warnUserAppUpdateRequired;

    this.EnabledSections = undefined; // Lista de secciones habilitadas
    this.callbacksSectionChange = []; // Callback cuando cambian las secciones
    this.effectiveEnabledSections();
}


/**
 * Verifica si hay conexión a internet.
 * En el caso de no haber conexion, la primera vez, invoca al handler networkFailureAlert
 *
 * @return una promesa resuelta si, se estima, que hay conexion, o rechazada en caso contrario.
 */
Nire.Connection.prototype._checkNetworkStatus = function() {
    var deferred = $.Deferred();
    if(navigator.network && navigator.network.connection.type == Connection.NONE) {
        deferred.reject({ status: "disabled_network" });
        this.networkFailureAlert();
    }
    else {
        deferred.resolve();
    }

    return deferred.promise();
}

/**
 * Retorna un objeto con los headers que requiere el request
 */
Nire.Connection.prototype._addHeaders = function() {
    var deviceId = "web"
    if(typeof device != "undefined")
        deviceId = device.uuid
    return { 
        "security-token": this.currentHash, 
        "security-user": this.currentUsername,
        "Accept-Language": LANG,
        "App-Domain": APP_DOMAIN,
        "App-TimezoneOffset": new Date().getTimezoneOffset(),
        "security-device": deviceId,
        "App-Version": APP_VERSION,
        "App-Build" : BUILD_VERSION_CODE
    };
};

/**
 * Realiza un request a una URL.
 *
 * Si data es NULL se realiza una petición GET, en caso contrario será un POST
 *
 * @param url url a la cual se hace el request.
 * @param data datos a enviar en el request.
 * @return promise que se resuelve cuando el servidor responde el request o da error.
 */
Nire.Connection.prototype._makeRequest = function(url, data, contentType, timeout, headers) {
    var self = this;

    var options = { headers: headers };
    options.method = data ? "POST" : "GET";
    options.dataType = "json";
    if(timeout) options.timeout = timeout;
    if(data) options.data = data;
    if(contentType) options.contentType = contentType;


    return $.ajax(url, options).done(function() { self.networkFailureWarned = false; });
};

/**
 * Verifica que la respuesta del servidor sea de una version compatible.
 * Si el servidor es más moderno, llama al handler de actulizar version.
 *
 * @param data respuesta del servidor
 * @return el mismo data, para que pueda seguir procesandose la llamada.
 */
Nire.Connection.prototype._checkClientVersion = function(data) {
    if(data.meta && !this.appUpdateRequiredWarned) {
        var serverVersion = data.meta.version
        if(serverVersion.split('.')[0] > Math.floor(SERVER_VERSION)) {
            this.warnUserAppUpdateRequired();
            this.appUpdateRequiredWarned = true;
        }
    }

    return data;
};

/**
 * Interpreta el error que un request puede retornar y actúa en consecuencia.
 */
Nire.Connection.prototype._handleRequestError = function(url, result, textStatus) {
    // Si el error fue debido a no haber red, no hacemos nada porque
    // ya fue manejado en _checkNetworkStatus
    if(result.status === "disabled_network") {
        return result;
    }

    // Si la url es la de CHECK_STATUS no es prioritaria y si produce errores no la revisamos
    // ademas que trae problemas con el frontweb en el login
    else if(url == URL_USER_CHECK_UPDATE) {
        return result;
    }

    else if(result.status === 401 || result.status === 403) {
        if(this.serverMonitor)
            this.serverMonitor.stop();
        
        this.notLoggedInCallback();
    }
    else if(textStatus === "timeout") {
        if(!this.networkFailureWarned) {
            this.networkFailureAlert();
            this.networkFailureWarned = true;
        }
    }
    else {
        if(Nire.analytics) {
            Nire.analytics.error("Error " + result.status + " " + textStatus + ", to url " + url + " User " + this.currentUsername + " Domain " + APP_DOMAIN + " Build" + BUILD_VERSION_CODE);
        }
        alertServerFailure();
    }
}

/**
 * Si el sevidor retornó informacion el el meta lo procesa
 */
Nire.Connection.prototype._processMeta = function(data) {
    if(data.meta && this.serverMonitor) {
        this.serverMonitor.executeOnDemand(data.meta);
    }
    return data;
}

/**
 * Realizar una llamada ajax al servidor controlando con conocimiento de los tokens de autenticacion
 * de ser necesarios.
 *
 * Si el parametro data es (false) se realizara una llamada GET, en caso contrario una llamada POST
 * Se espera que el servidor retorne siempre una respuesta JSON.
 */
Nire.Connection.prototype._serverCall = function(url, data, contentType, timeout) {
    var self = this;

    return this._checkNetworkStatus()
        .then( this._addHeaders.bind(this) )
        .then( this._makeRequest.bind(this, url, data, contentType, timeout) )
        .then( this._checkClientVersion.bind(this) )
        .then( this._processMeta.bind(this) )
        .fail( this._handleRequestError.bind(this, url) );
}

Nire.Connection.prototype._saveIfValid = function(key, value) {
    if(value.error && value.error.code == 0)
        this.cacheHandler.save(key, value);
    else if(value.message == "Success")
        this.cacheHandler.save(key, value);
}

Nire.Connection.prototype.registerDevice = function(deviceToken, platform, deviceId) {
    var str_url = String.format(URL_REGISTRY_DEVICE);
    return this._serverCall(str_url, { deviceToken: deviceToken, platform: platform, deviceId: deviceId });
}

/**
 * Recuperar la pagina home
 */
Nire.Connection.prototype.home = function(date, timezone) {
    var self = this;
    var str_url = String.format(URL_PAGE_HOME, date.format("YYYYMMDD"));
    return self._get(str_url, {timezone: timezone}).done(function(data){
        if(data.error.code == 0){
            if(!PREFETCH) return

            self._getDayCompleteData(data, date);
            self._getHealthData()

            // Realizamos las llamadas de los dias siguientes para cargar el cache
            var cacheDay = moment(date);
            self._get( String.format(URL_PAGE_HOME, cacheDay.add('days', 1).format("YYYYMMDD")) );
            self._get( String.format(URL_PAGE_HOME, cacheDay.add('days', -2).format("YYYYMMDD")) );
        }
    })
}

Nire.Connection.prototype.timeline = function(date) {
    var str_url = String.format(URL_PAGE_TIMELINE, date.format("YYYYMMDD"));
    var data = {notificationId: Nire.notification.getIdLastNotification()};
    return this._get(str_url, data);
}

Nire.Connection.prototype.timeslots = function(date) {
    var str_url = String.format(URL_PAGE_TIMESLOTS);
    return this._get(str_url);
}

Nire.Connection.prototype._getHealthData = function() {
    var _this = this;
    var queue = new Nire.AsyncQueue();

    _this.health().done(function(data){
        $.each(data.data.body.indicators, function(index, item) {
            queue.add( _this.indicator.bind(_this, item.type) );            
        });
    });
}

Nire.Connection.prototype._getDayCompleteData = function(data, date){
    var _this = this;
    var queue = new Nire.AsyncQueue();
    var dateString = date.format("YYYYMMDD");

    // Solicitmaos el detalle de todas las dietas y ejercicios    
    $.each(data.data.body.tracks, function(index, value) {
        if(isNaN(value.id)) return;
        if(value.section === "physicalActivity") {
            queue.add( _this.physicalActivityDetail.bind(_this, value.id, dateString) );
            queue.add( _this.physicalActivitySteps.bind(_this, value.id, dateString) );
        }
        if(value.section === "nutrition") {
            queue.add( _this.nutritionDetail.bind(_this, value.id, dateString) );
        }
    });

    // solicitamos otras paginas relacionadas
    queue.add( _this.nutritionWeekly.bind(_this, date) );
    queue.add( _this.physicalActivityWeekly.bind(_this, date) );
}

/**
 * Recuperar una petición desde el cache, o realizarla contra el servidor si no
 * se encuentra disponible en el cache.
 */
Nire.Connection.prototype._get = function(str_url, parameters, contentType){
    var self = this;

    var promise = $.Deferred();

    var cachekey = str_url + (parameters && parameters.day? parameters.day : "");
    var cacheData = self.cacheHandler.get(cachekey);
    if(cacheData) {
        promise.resolve(cacheData);
    }
    else {
        promise = self._serverCall(str_url, parameters, contentType).done(function(json) {
            var cacheKey = str_url;
            if(parameters && parameters.day) cacheKey += parameters.day;
            self._saveIfValid(cacheKey, json);
        });
    }

    return promise;
}

Nire.Connection.prototype.registryBiography = function(registryActivity, date, gender, height, weight){
    var str_url = String.format(URL_REGISTRY_BIOGRAPHY);
    var data = {registryActivity: registryActivity, date: date, gender: gender, height: height, weight: weight};
    return this._serverCall(str_url, data);
}

Nire.Connection.prototype.obtainBiography = function() {
    var str_url = String.format(URL_OBTAIN_BIOGRAPHY);
    return this._serverCall(str_url);
}

/**
 * Recuperar la pagina perfil de usuario
 */
Nire.Connection.prototype.userProfile = function() {
    var str_url = String.format(URL_USER_PROFILE);
    return this._serverCall(str_url);
}

/**
 * Recuperar la información que ofrece el asistente personal
 */
Nire.Connection.prototype.personalAssistant = function(section) {
    var str_url = String.format(URL_PERSONAL_ASSISTANT);
    return this._serverCall(str_url, {section: section});
}

/**
 * Recuperar el menu
 */
Nire.Connection.prototype.menu = function() {
	var str_url = String.format(URL_MENU_HOME);
	var promise = this._get(str_url);
	// Interceptamos algunos datos del  menu para tenerlos disponibles
	// para diferentes paginas.
	promise.done(function(json) {
		Nire.rangeData = {
			minDate: moment.utc(json.data.rangeData.minDate),
			maxDate: moment.utc(json.data.rangeData.maxDate)
		}
	});

	return promise;
}

/**
 * Recuperar la informacion nutricional
 */

//Pantalla home de nutrición
Nire.Connection.prototype.nutritionHome = function() {
	var str_url = String.format(URL_NUTRITION_HOME);
    return this._get(str_url);
}

Nire.Connection.prototype.nutritionWeekly = function( day ) {
	var str_url = String.format(URL_NUTRITION_WEEKLY, day.format("YYYYMMDD"));
	return this._get(str_url);
}

Nire.Connection.prototype.nutritionDetail = function( info, date ) {
	var str_url = String.format(URL_NUTRITION_DETAIL,  info);
	return this._get(str_url, {date: date});
}

Nire.Connection.prototype.saveRate = function( id, rate) {
    this.invalidateCache();
    return this._serverCall(URL_NUTRITION_RATE, { id: id, rate: rate });
}

Nire.Connection.prototype.nutritionHelp = function(){
    var str_url = String.format(URL_NUTRITION_HELP);
    return this._get(str_url);
}

Nire.Connection.prototype.health = function() {
    var str_url = String.format(URL_HEALTH_INDICATOR);
    return this._get(str_url);
}

Nire.Connection.prototype.healthTreatmentDetail = function(id) {
	var str_url = String.format(URL_HEALTH_TREATMENT_DETAIL, id);
	return this._serverCall(str_url);
}

Nire.Connection.prototype.healthTreatments = function() {
	var str_url = String.format(URL_HEALTH_TREATMENT);
	return this._get(str_url);
}

Nire.Connection.prototype.healthHelp = function(){
    var str_url = String.format(URL_HEALTH_HELP);
    return this._get(str_url);
}

Nire.Connection.prototype.healthNew = function(id, date, quantity, time, notificationId) {
    this.invalidateCache();
    var data = { id: id, date: date, quantity: quantity, time: time };
    if(notificationId)
        data.notificationId = notificationId;
    return this._serverCall(URL_HEALTH_NEW, data);
}

Nire.Connection.prototype.medicineNew = function(id, quantity, date, time) {
    this.invalidateCache();
    return this._serverCall(URL_MEDICINE_NEW, { id: id, date: date, quantity: quantity, time: time });
}

Nire.Connection.prototype.physicalActivityHelp = function(){
    var str_url = String.format(URL_PHYSICAL_ACTIVITY_HELP);
    return this._get(str_url);
}

Nire.Connection.prototype.physicalActivityWeekly = function(day) {
	var str_url = String.format(URL_PHYSICAL_ACTIVITY_WEEKLY, day.format("YYYYMMDD"));
	return this._get(str_url);
}

Nire.Connection.prototype.physicalActivityDetail = function( info, day ) {
    var str_url = String.format(URL_PHYSICAL_ACTIVITY_DETAIL, info);
    return this._get(str_url, {day: day});
}

Nire.Connection.prototype.physicalActivitySteps = function( info ) {
    var str_url = String.format(URL_PHYSICAL_ACTIVITY_STEPS, info);
    return this._get(str_url);
}

Nire.Connection.prototype.deviceWearables = function() {
    var str_url = String.format(URL_DEVICE_WEARABLES);
    return this._serverCall(str_url, {user: this.currentUsername, domain: APP_DOMAIN}, "", 0);
}

Nire.Connection.prototype.synchronizeApp = function(refreshUrl, name) {
    var str_url = String.format(URL_APP_REFRESH);
    return this._serverCall(str_url, {refreshUrl: refreshUrl, name: name}, "", 0);
}

Nire.Connection.prototype.newTrackHealthMedicine = function(searchString) {
    var str_url = String.format(URL_NEW_TRACK_HEALTH_MEDICINE).toLowerCase().replace("-", "_");

    // No utilizamos ServerCall para esta llamada porque los requerimientos para elasticSearch
    // son diferentes a los del resto de la application (content-type, headers, ...)
    var options = { 
        type: "GET", 
        dataType: "json", 
        contentType: "application/json"
    };

    return $.ajax(str_url + "?q=name:*"+ searchString +"*&fields=id,name&sort=_score:asc",  options)
}

Nire.Connection.prototype.newTrackHealthHeaders = function(day) {
    var str_url = String.format(URL_NEW_TRACK_HEALTH_HEADERS, day.format("YYYYMMDD"));
    return this._serverCall(str_url);
}

/**
 * Información utilizada para mostrar la pantalla de nuevo track de medicina.
 * Continene lista de indicadores de salud más utilizados y actuales.
 * Tambien contiene la lista de indicadores de medicamentos mas utilizados.
 */
Nire.Connection.prototype.newHealth = function(day) {
    var str_url = String.format(URL_NEW_TRACK_HEALTH, day.format("YYYYMMDD"));
    return this._serverCall(str_url);
}

Nire.Connection.prototype.newPhysicalActivity = function(day) {
    var str_url = String.format(URL_NEW_TRACK_PHYSICAL_ACTIVITY, day.format("YYYYMMDD"));
    return this._serverCall(str_url);
}

Nire.Connection.prototype.newNutrition = function(day) {
    var str_url = String.format(URL_NEW_TRACK_NUTRITION, day.format("YYYYMMDD"));
    return this._serverCall(str_url);
}

// TODO: Verificar si se sigue utilizando
Nire.Connection.prototype.newPhysicalActivityHeaders = function(day) {
    var str_url = String.format(URL_NEW_TRACK_PHYSICAL_ACTIVITY_HEADERS, day);
    return this._serverCall(str_url);
}

// el usuario hizo una actividad fisica fuera de su planificación
Nire.Connection.prototype.physicalActivityNew = function(id, date, time) {
    this.invalidateCache();
    return this._serverCall(URL_PHYSICAL_ACTIVITY_NEW, { id: id, date: date, time: time });
}

// Pantalla home de actividad fisica
Nire.Connection.prototype.physicalActivityHome = function() {
	var str_url = String.format(URL_PHYSICAL_ACTIVITY_HOME);
    return this._get(str_url);
}

// el usuario comió algo fuera de su planificación
Nire.Connection.prototype.nutritionNew = function(id, date, meal, quantity) {
    this.invalidateCache();
    return this._serverCall(URL_NUTRITION_NEW, { id: id, date: date, meal: meal, quantity: quantity });
}

/**
 * Agrega un plato al Timeline
 *
 * @param (long) idMeal identificador de la hora del día (almuerzo, cena, etc.)
 * @param (array<map>) array de mapas con la información de cada plato
 *
 * Por los problemas del asincronismo y el no querer consumir más memoria que la necesaria
 * el servidor no tien la información de todos plos platos que se podrian agregar.
 *
 * Entonces este método utiliza "extras" para enviarle al servidor conocimiento previo de
 * todos los datos necesario. Luego cuando el servidor tenga tiemp recuperará la información
 * correcta y completa, pero al menos así podremos mostrar el home de la aplicacion inmediatamente.
 */
Nire.Connection.prototype.addNutrition = function(date, idMeal, plates) {
    this.invalidateCache();
    var data = { date: date, plates: plates, idMeal: idMeal }
    return this._serverCall(URL_NUTRITION_ADD, JSON.stringify(data), "application/json");
}

Nire.Connection.prototype.indicator = function(info) {
	var str_url = String.format(URL_INDICATOR_DETAIL,  info);
    return this._get(str_url);
}

Nire.Connection.prototype.getPreguntasPendientes = function() {
	return this._serverCall(URL_PREGUNTAS_PENDIENTES);
}

Nire.Connection.prototype.getPreguntasRespondidas = function() {
    return this._serverCall(URL_PREGUNTAS_RESPONDIDAS);
}

Nire.Connection.prototype.getPreguntas = function() {
	return this._serverCall(URL_PREGUNTAS);
}

Nire.Connection.prototype.getPreguntaNueva = function(id) {
	var str_url = String.format(URL_PREGUNTA_NUEVA,  id);
	return this._serverCall(str_url);
}

/**
 * Envia al servidor la respuesta a una preugnta de perfil
 */
Nire.Connection.prototype.respuesta = function(idPregunta, valor, notificationId) {
    this.invalidateCache();

    // Si es un array de objetos enviamos un json, en caso contrario 
    // lo guaramos como strings seprados por comas.
    if(Array.isArray(valor)) {
        if(valor.length > 0 && typeof valor[0] === 'object') {
            valor = JSON.stringify(valor)
        }
        else {
            valor = valor.join(",");
        }
    }

    var data = { id: idPregunta, valor: valor };

    if(notificationId) data.notificationId = notificationId;
	return this._serverCall(URL_RESPUESTA, data);
}

Nire.Connection.prototype.generarDieta = function(){
    return this._serverCall(URL_GENERAR_DIETA);
}

Nire.Connection.prototype.regenerateDietPetiton = function(){
    this._serverCall(URL_REGENERAR_DIETA);
}

/**
 * Solicita al servidor borrar un track de nutricion
 */
Nire.Connection.prototype.deleteNutritionTrack = function(day, idMeal, idTrack) {
    if(moment.isMoment(day)) day = day.format("YYYYMMDD");

    this.invalidateCache();
    return this._serverCall(
        URL_NUTRITION_DELETE, 
        { date: day, idMeal: idMeal, idTrack: idTrack }
    );
}

Nire.Connection.prototype.updateTrack = function(day, idTrack, section, type) {
    if(section != "treatment") {
        if(day.isAfter())
        {
            var promise = $.Deferred();
            promise.resolve({error: {code:1, message: "DATE_AFTER_TODAY"}});
            return promise;
        }
    
        day = day.format("YYYYMMDD");
    }

    // No importa como llegamos, si la fecha no es un String, la transformamos.
    if(moment.isMoment(day)) day = day.format("YYYYMMDD")

    this.invalidateCache();
    return this._serverCall(URL_UPDATE_TRACK, {
    	day: day,
    	idTrack: idTrack,
    	section: section,
    	type: type,
        timezone: new Date().getTimezoneOffset()
    });
}

Nire.Connection.prototype.getFood = function(category) {
    return this._serverCall(String.format(URL_NEW_TRACK_NUTRITION_FOOD, category));
}

Nire.Connection.prototype.appHelp = function(width, height, density) {
    var data = { width: Math.round(width), height: Math.round(height), pixelDensity: density };
	return this._serverCall(String.format(URL_APP_HELP), data);
}

Nire.Connection.prototype.pillBox = function( day, timezone ) {
	var str_url = String.format(URL_PILLBOX, day.format("YYYYMMDD"));
	return this._get(str_url, {timezone: timezone});
}

Nire.Connection.prototype.clinicHistory = function( day, timezone ) {
	var str_url = String.format(URL_CLINIC_HISTORY, day.format("YYYYMMDD"));
	return this._get(str_url, {timezone: timezone});
}

Nire.Connection.prototype.clinicHistoryExplorer = function( day, timezone ) {
	var str_url = String.format(URL_CLINIC_HISTORY_EXPLORER, day.format("YYYYMMDD"));
	return this._get(str_url, {timezone: timezone});
}

Nire.Connection.prototype.clinicHistoryExplorerDetail = function( day, timezone, id ) {
	var str_url = String.format(URL_CLINIC_HISTORY_EXPLORER_DETAIL, day.format("YYYYMMDD"), id);
	return this._get(str_url, {timezone: timezone});
}

Nire.Connection.prototype.retrieveNotifications = function(idLastNotification) {
    // esta llamada no es cheada.
    var str_url = String.format(URL_NOTIFICATIONS, idLastNotification);
    return this._serverCall( str_url ).then(function(json) { return json.data });
}

Nire.Connection.prototype.checkNotifications = function(idLastNotification) {
    // esta llamada no es cheada.
    var str_url = String.format(URL_NOTIFICATIONS_COUNT, idLastNotification);
    return this._serverCall( str_url ).then(function(json) { return json.data });
}

Nire.Connection.prototype.replyNotification = function(answer, answerText, notificationId) {
    var str_url = URL_REPLY_NOTIFICATION;
    var data = {
        answer: answer, 
        text: answerText, 
        notificationId: notificationId
    }
    return this._serverCall(str_url, data);
}




/**
 * Información de un tratamiento específico
 * @param idTreatment identificador del tratamiento que se solicita informacion
 * @param day fecha que se quiere visualizar
 *
 * @return Información genérica del tratamiento y pauta diaria.
 */
Nire.Connection.prototype.treatment = function(idTreatment, day) {
    if(!day || !idTreatment) throw "Faltan parametros necesarios en la llamada a Connection::Treatment"
    var str_url = String.format(URL_GET_TREATMENT, idTreatment);
    if(moment.isMoment(day)) day = day.format("YYYYMMDD");

    return this._get( str_url, { day: day } ).then(function(json) { return json.data });
}

Nire.Connection.prototype.treatmentList = function() {
    var str_url = String.format(URL_GET_TREATMENT_LIST);
    return this._get( str_url ).then(function(json) { return json.data });
}

Nire.Connection.prototype.sendEmail = function(username) {
    var str_url = String.format(URL_SEND_EMAIL);
    return this._serverCall(str_url, {username: username});
}

/**
 * Intentar loguear un usuario en el sistema.
 *
 * Retorna una promesa que se cumple cuando el servidor informa que el usuario y la contraseña suministrada
 * son validos, y fracasa cuando se produce un error de comunicación con el servidor o los datos suministrados
 * no son validos.
 *
 * En el caso de fracaso, se invoca al fail con un parametro INVALID_CREDENTIALS si el motivo fue error en los
 * datos suministrados o SERVER_ERROR si fue un error de comunicación
 */
Nire.Connection.prototype.login = function(username, password, doctype) {
	var self = this;
	var data = { username: username, password: password, doctype: doctype };
	var str_url = String.format(URL_LOGIN);

	var promise = $.Deferred();

    // Si estamos en modo de desarrollo local (con json estaticos) la única validación
    // es que el username sea igual al password
    if (ENTORNO == "DESARROLLO") {
        if (username && password && username == password) {
            promise.resolve({redirect: "timeline"});
        }
        else {
            promise.reject();
        }
        return promise;
    }
	
	this._serverCall(str_url, data, "", 10000)
		.done(function(data) {
			if(data.hash) {
                self.configure( data.config );
                if(data.assignedUsername)
                    username = data.assignedUsername;
				self.initialize(data.hash, username);

                self.sendClientTokenIfAvailable();
				promise.resolve(data);
			}
			else {
				promise.reject(this, data, "INVALID_CREDENTIALS");
			}
		})
		.fail(function(result) {
			alertServerFailure();
			promise.reject(this, data, "SERVER_ERROR");
		});

	return promise;
}

/**
 * Intenta loguearse con las credenciales que estan guardadas. Si lo consigue
 * configura la instancia como logueada, en caso de no poder, borra las credenciales.
 *
 * Retorna un defered que se resuelve si se verifica al usuario logueado
 * o falla si no se puede verificar
 */
Nire.Connection.prototype.tryPreviousLogin = function() {
    var self = this;
    
    this.initialize();
    var deferred = $.Deferred();

    this._serverCall(URL_TRY_LOGIN)
        .done(function(data) { 
            self.configure( data.config );
            deferred.resolve(data) 
        })
        .fail(function() { deferred.reject(); })

    return deferred.promise();
}

/**
 * Intenta realizar un login por medio de un token de autenticacion
 * @param username nombre de usuario
 * @param token codigo con el cual intenta loguearse.
 */
Nire.Connection.prototype.tokenLogin = function(username, token) {
    var self = this;

    var data = { username: username, token: token };
    return this._serverCall(URL_TOKEN_LOGIN, data).done(function(data) {
        self.configure( data.config );
        if(data.assignedUsername)
            username = data.assignedUsername;
        
        self.initialize(data.hash, username);
    });
}

Nire.Connection.prototype.configureSections = function () {
    var self = this;
    var str_url = String.format(URL_ENABLED_SECTIONS);
    return this._serverCall(str_url).done(function(data) {
        self.configure( data.config );
    });
}

/**
 * Si desde el servidor recibimos configuracion en caliente la aplicamos. Algunos cambios de configuracion
 * serán simplemente pisar avariables, otros cambios requerirán ejecutar más opciones.
 **/
Nire.Connection.prototype.configure = function( configuration ) {
    if(!configuration) return;
    if(configuration.replace) this.replaceGlobalNamespace(configuration.replace);
    this.configureEnabledSections(
        this.effectiveEnabledSections(configuration.allow, configuration.deny)
    );
}

/**
 * Se recibe un mapa. Con este mapa sobreescribimos o creamos variables en global namespace
 **/
Nire.Connection.prototype.replaceGlobalNamespace = function( map ) {
    for (var key in map) {
        window[key] = map[key];
    }
}

/**
 * Recibe la lista de secciones habilitadas y deshabilitdas, y junto con los valores default
 * las combina para armar una lista absoluta de secciones habilitadas
 */
Nire.Connection.prototype.effectiveEnabledSections = function( allow, deny ) {
    if(!deny) deny = [];
    if(!allow) allow = [];

    function notIn( array ) { return function(item) { return array.indexOf(item) == -1 } };

    var disabled = DISABLED_SECTIONS.filter( notIn(allow) );
    if( allow.indexOf("*") >= 0) disabled = [];

    var result = KNOWN_SECTIONS.filter( notIn(disabled) );
    result = result.filter( notIn(deny) );

    // Excepto la primera vez, verificamos is hubieron realmente cambios
    // en las secciones antes de notificar a los callbacks
    var changes = !this.EnabledSections || this.EnabledSections.length != result.length;
    if(!changes) changes = this.EnabledSections.some(function(item) {
        return result.indexOf(item) === -1;
    });
    this.EnabledSections = result;

    if(changes)
        this.callbacksSectionChange.forEach(function(callback) { callback(result) });

    return result;
}

/**
 * Devuelve la lista de secciones estan habilitadas actualmente.
 */ 
Nire.Connection.prototype.getEnabledSections = function(){
    if(typeof this.EnabledSections === "undefined") this.effectiveEnabledSections();    
    return this.EnabledSections;
}

/**
 * Informa si una sección en particular esta habilitada o no
 */
Nire.Connection.prototype.isEnabledSection = function(section) {
    return this.getEnabledSections().indexOf(section) >= 0;
}

/**
 * Agrega un callback que sera notificado cuando la lista de secciones habilitadas cambie.
 */
Nire.Connection.prototype.onSectionChange = function(callback) {
    this.callbacksSectionChange.push(callback);
}

/**
 * Recibe una lista con las secciones que deben estar habilitadas y configura el sistema.
 */
Nire.Connection.prototype.configureEnabledSections = function( sections ) {
    function notIn( array ) { return function(item) { return array.indexOf(item) == -1 } };

    // Generamos un TAG style y lo agregamos al DOM con un ID conocido
    // para poder eliminarlo ants de insertar uno nuevo
    var toHide = KNOWN_SECTIONS.filter( notIn(sections) );

    var cssScript = toHide.reduce(function(previous, current) {
        return previous + ".SECTION_" + current + " { display: none !important }\n";
    }, "");

    $("#__HIDDEN_SECTIONS_CSS__").remove();
    $("head").append("<style type='text/css' id='__HIDDEN_SECTIONS_CSS__'>" + cssScript + "</style>");
};

/**
 * Inicializa la clase para poder realizar consultas al servidor con las credenciales correctas.
 * Si hash y username son vacios, utiliza los de la ultima conexión
 */
Nire.Connection.prototype.initialize = function(hash, username) {
    if(hash == null && username == null) {
        username = localStorage.getItem("LAST_USERNAME");
        hash = localStorage.getItem("LAST_HASH");
    }
    localStorage.setItem("LAST_USERNAME", username);
    localStorage.setItem("LAST_HASH", hash)
    
    this.currentHash = hash;
    this.currentUsername = username;
    
    Nire.localStorage = new Nire.LocalStorage(username, APP_DOMAIN);
    WEB_APP = false;
    //if (!WEB_APP) {
        this.cacheHandler = new Nire.CacheHandler(username);
        this.cacheInvalidator = new Nire.CacheInvalidationObserver(this)
        // Creo y configuro ServerMonitor para actualizar el estado de app
        // de acuerdo indicadiones del servidor periodicamente.
        this.serverMonitor = new Nire.ServerMonitor(this);
        this.serverMonitor.addListener( this.cacheInvalidator );
        this.serverMonitor.addListener( new Nire.NotificationObserver(Nire.notification) );
        this.serverMonitor.start()
    //}
};

/**
 * Desloguea al usuario actual
 */
Nire.Connection.prototype.logout = function() {
    localStorage.setItem("LAST_HASH", "");

    this.currentHash = "";
    this.currentUsername = "";
    this.localStorage = null;
};

/**
 * Enviar al servidor el token que identifica al dispositivo cliente para notificaciones push.
 * si no hay un token registrado, o no se conoce el usuario, este método no hace nada.
 */
Nire.Connection.prototype.sendClientTokenIfAvailable = function() {
    var self = this;
    var deviceToken = localStorage.getItem("deviceReg_token");
    if(deviceToken) {
        this.registerDevice(deviceToken, device.platform, device.uuid)
            .done(function() { localStorage.removeItem("deviceReg_token") })
            .fail(function() { self.log("Error when registering client token"); });
    }
    else {
        this.log("Client token not available");
    }
}

Nire.Connection.prototype.forgottenPassword = function(username){
    return this._serverCall(String.format(URL_PASSWORD_RECOVER, username));
}

/**
 * Conectarse con el servidor y recuperar la información de stauts y demas comandos que tenga para
 * el usuario actualmente logueado. Este método lo utiliza ServerMonitor
 */
Nire.Connection.prototype.getServerStatus = function(){
    return this._serverCall(URL_USER_CHECK_UPDATE);
}

/**
 * Si estamos manejando un cache, lo invalidamos completamente.
 */
Nire.Connection.prototype.invalidateCache = function() {
    if(this.cacheHandler)
        this.cacheHandler.invalidate();
}

Nire.Connection.prototype.getMedWhatSession = function() {
    var data = {email: MEDWHAT_EMAIL, password: MEDWHAT_PASSWORD}
    return this._get(URL_MEDWHAT_SESSION, JSON.stringify(data), "application/json").done(function(a){console.log(a)}).fail(function(e){console.log(e)})
}

Nire.Connection.prototype.medWhatConsultor = function(session, query){
    var data = {question: query, session: session, passcode: MEDWHAT_PASSCODE};
    console.log(data)
    return this._serverCall(URL_MEDWHAT_QUESTION, JSON.stringify(data), "application/json");
}

Nire.Connection.prototype.saveMedWhatLog = function(question, answer) {
    this._serverCall(URL_SAVE_MEDWHAT_LOG, {question: question, answer: answer});
}
