describe('Profile Questions', function () {
    var searchService;
    var sut;

    beforeEach(function() {
        $("#scrap").html("<h1 id='title'></h1><div id='body'></div>")

        var deferred = $.Deferred();
        deferred.resolve([{ _source: { label: 'result 1'} }, {_source: { label: 'result 2'}}, {_source: { label: 'result 3'}}]);
        searchService = { search: function(index, query) { return deferred.promise(); } };

        sut = new Nire.ProfileQuestions(searchService, {
            labelPlaceholder: "#title",
            bodyPlaceholder: "#body",
        });
    }); 

    describe('Text Questions', function() {
        var question;

        beforeEach(function() {
            question = {
                label: 'This is a text question',
                type: 'text'
            };
        });

        it("Se soportan preguntas con respuesta de texto libre", function() {
            sut.show(question);

            expect($("#title")).to.have.text("This is a text question");
            expect($("#body input")).to.have.attr("type", "text");
        });

        it("Se inicializan con un valor", function() {
            question.value = "valor ingresado";
            sut.show(question);

            expect($("#body input")).have.value("valor ingresado");
        });

        it("se puede consultar el valor cuando cambia", function() {
            sut.show(question);

            $("#body input").val("testing").trigger("input")
            expect(sut.currentQuestion.value).to.be.equals("testing");
        })
    });

    describe("Checkbox Questions", function() {
        var question;

        beforeEach(function() {
            question = {
                label: 'This is a checkbox question',
                type: 'checkbox',
                options: [
                    { label: 'opcion 1', value: 'opc1' },
                    { label: 'opcion 2', value: 'opc2' },
                ]
            };
        });

        it("Se soportan preguntas con respuesta multiple", function() {
            sut.show(question);

            expect($("#title")).to.have.text("This is a checkbox question");
            expect($("#body input[type=checkbox]"))
                .to.have.length(2)
                .and.to.have.attr("value", "opc1");
        });

        it("se inicializan con un valor", function() {
            question.value = ["opc2"];

            sut.show(question);
            expect($("#body input:checked")).to.have.length(1)
        });

        it("se puede consultar el valor cuando cambia", function(done) {
            sut.show(question);

            $("#body input").prop("checked", true).change();
            setTimeout(function() {
                expect(sut.currentQuestion.value).to.be.eqls(["opc1", "opc2"]);
                done();
            }, 1);
        })

    });

    describe("Options Questions", function() {
        var question

        beforeEach(function() {
            question = {
                label: 'This is a options question',
                type: 'option',
                options: [
                    { label: 'opcion 1', value: 'opc1' },
                    { label: 'opcion 2', value: 'opc2' },
                ]
            };
        });

        it("Se soportan preguntas con opciones para elegir una", function() {
            sut.show(question);

            expect($("#title")).to.have.text("This is a options question");
            expect($("#body input[type=radio]"))
                .to.have.length(2)
                .and.to.have.attr("value", "opc1");
        });

        it("se inicializan con un valor", function() {
            question.value = "opc2";

            sut.show(question);
            expect($("#body input:checked")).to.have.value("opc2")
        });

        it("se puede consultar el valor cuando cambia", function() {
            sut.show(question);

            $("#body input[value='opc2']").prop("checked", true).change();
            expect(sut.currentQuestion.value).to.be.equals("opc2");
        });
    });

    describe("Elasticsearch Questions", function() {
        var question;

        beforeEach(function() {
            question = {
                label: 'This is a elasticsearch question',
                type: 'elasticsearch',
                index: 'elasticsearch_index',
                queryTemplate: "nombredieta:#term#"
            }
        });

        it("Se soportan preguntas de eleccion multiple contra elasticsearch", function() {
            sut.show(question);

            expect($("#body input"))
                .to.have.length(1)
                .and.to.have.attr("type", "text");
        });

        it("Al escribe se consulta elasticsearch y muestran los resultados", function(done) {
            var originalFnc = sut.showSearchResults.bind(sut);
            var stub = sinon.stub(sut, "showSearchResults", function(data) { return originalFnc(data); })

            sut.show(question);
            $("#body input").val("busqueda").trigger("input");

            setTimeout(function() {
                expect($("#body input"), "El valor de la busqueda debe preservarse").have.prop("value", "busqueda");
                expect($("#results .result")).have.length(3);
                done();
            }, 250)

        });

        it("Se si escribe muy rapido solo se llama a search una unica vez", function(done) {
            searchService.search = sinon.mock()
            searchService.search.once().returns( $.Deferred() )

            sut.show(question);
            $("#body input").val("b").trigger("input");
            $("#body input").val("bu").trigger("input");
            $("#body input").val("bus").trigger("input");

            setTimeout(function() {
                searchService.search.verify();
                done();
            }, 250)
        });

        it("La busqueda se realiza con el indice, y el template de query de la pregunta", function(done) {
            searchService.search = sinon.mock()
            searchService.search.once().withExactArgs("elasticsearch_index", "nombredieta:b*").returns( $.Deferred() )

            sut.show(question);
            $("#body input").val("b").trigger("input");

            setTimeout(function() {
                searchService.search.verify();
                done();
            }, 250)
        });

        it("Se pueden seleccionar varios items", function(done) {
            sut.show(question);
            $("#body input").val("b").trigger("input");

            setTimeout(function() {
                $($("#results .result")[0]).click();
                $($("#results .result")[2]).click();

                expect(sut.currentQuestion.value).to.be.eqls([{label: 'result 1'},{ label: 'result 3'}])
                expect($("#selection .item")).to.have.length(2);
                done();
            }, 250)
        });

        it("se pueden deseleccionar items", function() {
            question.value = [{ label: 'option 1' }, { label: 'option 2'}];
            sut.show(question)
            expect($("#selection .item")).to.have.length(2);

            $($("#selection .item")[0]).click();
            expect($("#selection .item"), "El item debería borrarse").to.have.length(1);
            expect(sut.currentQuestion.value).to.be.eqls([{ label: 'option 2'}])
        });
    });
});
