describe('Nire.Connection#_serverCall', function () {
    var notLoggedInCallback, 
        networkFailureAlert, 
        warnUserAppUpdateRequired;

    beforeEach(function() {
        // Borramos el mock de que no hay conectividad.
        delete(navigator.network);

        notLoggedInCallback = sinon.spy();
        networkFailureAlert = sinon.spy();
        warnUserAppUpdateRequired = sinon.spy();

        // Mock para ajax
        this.xhr = sinon.useFakeXMLHttpRequest();
        var requests = this.requests  = [];
        this.xhr.onCreate = function(xhr) { requests.push(xhr); }
    });

    afterEach(function() {
        this.xhr.restore();
    });

    it('Si no hay conectividad se rechaza el request informando el error', function(done) {
        // Emulamos que no hay connectividad.
        navigator.network  = { connection: { type: Connection.NONE }};

        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);
        connection._serverCall('mock_url').fail(function(value) {
            assert.equal(value.status, "disabled_network");
            assert.ok(networkFailureAlert.called);
            done();
        });
    });

    it("si no se pasan parametros se realiza una peticion get", function() {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);
        connection._serverCall('mock_url');
        assert.equal(this.requests[0].method, "GET");
    });

    it("si se pasan parámetros se realiza una peticion post", function() {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);
        connection._serverCall('mock_url', { some: 'data' });
        assert.equal(this.requests[0].method, "POST");
        console.log(this.requests[0])
        assert.equal(this.requests[0].requestBody, "some=data")
    });

    it("se agregan los headers requeridos por nire", function() {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);

        // Simulamos que tenemos un username y token.
        connection.currentHash = "dummyToken";
        connection.currentUsername = "dummyUser";

        // Llamamos al SUT y verificamos los Headers
        connection._serverCall('mock_url');
        assert.equal(this.requests[0].requestHeaders["security-token"], "dummyToken");
        assert.equal(this.requests[0].requestHeaders["security-user"], "dummyUser");
        assert.equal(this.requests[0].requestHeaders["Accept-Language"], LANG);
        assert.equal(this.requests[0].requestHeaders["App-Domain"], APP_DOMAIN);
        assert.ok(this.requests[0].requestHeaders["App-TimezoneOffset"]);
    });

    it("el servidor puede informar que se debe actualizar el cliente", function() {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);
        connection._serverCall("mock_url")
        this.requests[0].respond(200, {}, '{"meta": {"version": "2.0"}}');

        assert.ok(warnUserAppUpdateRequired.called);
    });

    it("cambios de version menores en el servidor no producen una actualizacion", function() {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);
        connection._serverCall("mock_url")
        this.requests[0].respond(200, {}, '{"meta": {"version": "1.1"}}');

        sinon.assert.notCalled(warnUserAppUpdateRequired);
    });

    it("cuando el servidor informa sobre actualizacion dos veces solo se avisa una vez", function() {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);

        connection._serverCall("mock_url")
        connection._serverCall("mock_url")

        this.requests[0].respond(200, {}, '{"meta": {"version": "2.0"}}');
        this.requests[1].respond(200, {}, '{"meta": {"version": "2.0"}}');

        sinon.assert.calledOnce(warnUserAppUpdateRequired);
    });

    // Sería coherente este comportamiento, pero no funciona de esta forma.
    // Un error 500 en el servidor no es reportado por esta llamada.
    it("si se produce un error no esperado se le informa al cliente", function() {
        // TODO: 
        // Se requiere esta función en el global namespace.
        // ¿No debería usarse al misma que se utiliza para construir la app?
        // Lamentablemente como esta ahora, el mensaje que muestran es diferente.
        alertServerFailure = sinon.spy();

        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);
        connection._serverCall("mock_url")
        this.requests[0].respond(500, {}, 'error');

        sinon.assert.called(alertServerFailure);
        delete(alertServerFailure);
    });

    it("cuando servidor retorna 401 se indica que no se esta logueado", function() {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);
        connection.serverMonitor = { stop: sinon.spy() }

        connection._serverCall("mock_url")
        this.requests[0].respond(401, {}, '{}');

        sinon.assert.called(notLoggedInCallback);
    });

    it("cuando servidor retorna 403 se indica que no se esta logueado", function() {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);
        connection.serverMonitor = { stop: sinon.spy() }

        connection._serverCall("mock_url")
        this.requests[0].respond(403, {}, '{}');

        sinon.assert.called(notLoggedInCallback);
    });

    it("cuando una llamada hace timeout se le informa el usuario", function(done) {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);

        // El cuarto parámetro es el tiempo a esperar por una respuesta antes de timeout
        connection._serverCall("mock_url", null, null, 100).fail(function() { 
            sinon.assert.called(networkFailureAlert);
            done();
        });
    });
    it("dos timeouts consecutivos son reportados una única vez", function(done) {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);

        // El cuarto parámetro es el tiempo a esperar por una respuesta antes de timeout
        connection._serverCall("mock_url", null, null, 100).fail(function() {
            connection._serverCall("mock_url", null, null, 100).fail(function() {
                sinon.assert.calledOnce(networkFailureAlert);
                done();
            });
        });
    });
    it("una llamada exitosa vuelve a mostrar el alerta de timeout", function(done) {
        var self = this;
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);

        // El cuarto parámetro es el tiempo a esperar por una respuesta antes de timeout
        connection._serverCall("one", null, null, 100).fail(function() {
            connection._serverCall("two", null, null, 100).done(function() {
                connection._serverCall("three", null, null, 100).fail(function() {
                    sinon.assert.calledTwice(networkFailureAlert);
                    done();
                })
            });
            self.requests[1].respond(200, {}, '{}');
        });
    });
    it("si la llamada a userUpdated da error se ignora por ser poco prioritaria y afectar la web", function() {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);

        alertServerFailure = sinon.spy();
        connection.getServerStatus()

        this.requests[0].respond(500, {}, 'error');

        sinon.assert.notCalled(alertServerFailure);
        delete(alertServerFailure);
    });
    it("cuando el servidor responde con un meta y hay un serverMonitor se lo ejecuta", function() {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);
        connection.serverMonitor = { executeOnDemand: sinon.spy() }
   
        connection._serverCall("mock_url");
        this.requests[0].respond(200, {}, '{"meta": {"version": "2.0"}}');

        assert.ok(connection.serverMonitor.executeOnDemand.called);
    });
    it("se puede especificar el content-type del request", function() {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);
   
        connection._serverCall("mock_url", null, "application/json")
        assert.equal(this.requests[0].requestHeaders["Content-Type"], "application/json");
    });
    it("si todo sale bien se retorna la información del servidor", function(done) {
        var connection = new Nire.Connection(notLoggedInCallback, networkFailureAlert, warnUserAppUpdateRequired);
        connection._serverCall("mock_url").done(function(data) {
            assert.deepEqual(data, {meta: {version: "2.0"}});
            done();
        });

        this.requests[0].respond(200, {}, '{"meta": {"version": "2.0"}}');
    });
});